@@ -17,8 +17,12 @@ import os
 
 import yaml
 
+try:
+    import keystoneclient.auth as ksc_auth
+except ImportError:
+    ksc_auth = None
+
 from os_client_config import cloud_config
-from os_client_config import defaults_dict
 from os_client_config import exceptions
 from os_client_config import vendors
 
@@ -32,7 +36,6 @@ CACHE_PATH = os.path.join(os.path.expanduser(
     os.environ.get('XDG_CACHE_PATH', os.path.join('~', '.cache'))),
     'openstack')
 BOOL_KEYS = ('insecure', 'cache')
-REQUIRED_VALUES = ('auth_url', 'username', 'password')
 VENDOR_SEARCH_PATH = [os.getcwd(), CONFIG_HOME, '/etc/openstack']
 VENDOR_FILES = [
     os.path.join(d, 'clouds-public.yaml') for d in VENDOR_SEARCH_PATH]
@@ -44,31 +47,36 @@ def get_boolean(value):
     return False
 
 
+def _get_os_environ(defaults):
+    for (k, v) in os.environ.items():
+        if k.startswith('OS_'):
+            newkey = k[3:].lower()
+            defaults[newkey] = v
+    return defaults
+
+
+def _auth_update(old_dict, new_dict):
+    """Like dict.update, except handling the nested dict called auth."""
+    for (k, v) in new_dict.items():
+        if k == 'auth':
+            old_dict[k].update(v)
+        else:
+            old_dict[k] = v
+    return old_dict
+
+
 class OpenStackConfig(object):
 
     def __init__(self, config_files=None, vendor_files=None):
         self._config_files = config_files or CONFIG_FILES
         self._vendor_files = vendor_files or VENDOR_FILES
 
-        defaults = defaults_dict.DefaultsDict()
-        defaults.add('username')
-        defaults.add('user_domain_name')
-        defaults.add('password')
-        defaults.add(
-            'project_name', defaults.get('username', None),
-            also='tenant_name')
-        defaults.add('project_id', also='tenant_id')
-        defaults.add('project_domain_name')
-        defaults.add('auth_url')
-        defaults.add('region_name')
-        defaults.add('cache')
-        defaults.add('auth_token')
-        defaults.add('insecure')
-        defaults.add('endpoint_type')
-        defaults.add('cacert')
-        defaults.add('auth_type')
-
-        self.defaults = defaults
+        defaults = dict(
+            auth_plugin='password',
+            auth=dict(),
+            compute_api_version='1.1',
+        )
+        self.defaults = _get_os_environ(defaults)
 
         # use a config file if it exists where expected
         self.cloud_config = self._load_config_file()
@@ -87,12 +95,14 @@ class OpenStackConfig(object):
     def _load_config_file(self):
         for path in self._config_files:
             if os.path.exists(path):
-                return yaml.load(open(path, 'r'))
+                with open(path, 'r') as f:
+                    return yaml.safe_load(f)
 
     def _load_vendor_file(self):
         for path in self._vendor_files:
             if os.path.exists(path):
-                return yaml.load(open(path, 'r'))
+                with open(path, 'r') as f:
+                    return yaml.safe_load(f)
 
     def get_cache_max_age(self):
         return self._cache_max_age
@@ -115,10 +125,14 @@ class OpenStackConfig(object):
 
     def _get_base_cloud_config(self, name):
         cloud = dict()
-        if name in self.cloud_config['clouds']:
-            our_cloud = self.cloud_config['clouds'][name]
-        else:
-            our_cloud = dict()
+
+        # Only validate cloud name if one was given
+        if name and name not in self.cloud_config['clouds']:
+            raise exceptions.OpenStackConfigException(
+                "Named cloud {name} requested that was not found.".format(
+                    name=name))
+
+        our_cloud = self.cloud_config['clouds'].get(name, dict())
 
         # Get the defaults (including env vars) first
         cloud.update(self.defaults)
@@ -128,15 +142,15 @@ class OpenStackConfig(object):
             cloud_name = our_cloud['cloud']
             vendor_file = self._load_vendor_file()
             if vendor_file and cloud_name in vendor_file['public-clouds']:
-                cloud.update(vendor_file['public-clouds'][cloud_name])
+                _auth_update(cloud, vendor_file['public-clouds'][cloud_name])
             else:
                 try:
-                    cloud.update(vendors.CLOUD_DEFAULTS[cloud_name])
+                    _auth_update(cloud, vendors.CLOUD_DEFAULTS[cloud_name])
                 except KeyError:
                     # Can't find the requested vendor config, go about business
                     pass
 
-        cloud.update(our_cloud)
+        _auth_update(cloud, our_cloud)
         if 'cloud' in cloud:
             del cloud['cloud']
 
@@ -190,6 +204,53 @@ class OpenStackConfig(object):
         new_args.update(os_args)
         return new_args
 
+    def _find_winning_auth_value(self, opt, config):
+        opt_name = opt.name.replace('-', '_')
+        if opt_name in config:
+            return config[opt_name]
+        else:
+            for d_opt in opt.deprecated_opts:
+                d_opt_name = d_opt.name.replace('-', '_')
+                if d_opt_name in config:
+                    return config[d_opt_name]
+
+    def _validate_auth(self, config):
+        # May throw a keystoneclient.exceptions.NoMatchingPlugin
+        plugin_options = ksc_auth.get_plugin_class(
+            config['auth_plugin']).get_options()
+
+        for p_opt in plugin_options:
+            # if it's in config.auth, win, kill it from config dict
+            # if it's in config and not in config.auth, move it
+            # deprecated loses to current
+            # provided beats default, deprecated or not
+            winning_value = self._find_winning_auth_value(
+                p_opt, config['auth'])
+            if not winning_value:
+                winning_value = self._find_winning_auth_value(p_opt, config)
+
+            # if the plugin tells us that this value is required
+            # then error if it's doesn't exist now
+            if not winning_value and p_opt.required:
+                raise exceptions.OpenStackConfigException(
+                    'Unable to find auth information for cloud'
+                    ' {cloud} in config files {files}'
+                    ' or environment variables. Missing value {auth_key}'
+                    ' required for auth plugin {plugin}'.format(
+                        cloud=cloud, files=','.join(self._config_files),
+                        auth_key=p_opt.name, plugin=config['auth_plugin']))
+
+            # Clean up after ourselves
+            for opt in [p_opt.name] + [o.name for o in p_opt.deprecated_opts]:
+                opt = opt.replace('-', '_')
+                config.pop(opt, None)
+                config['auth'].pop(opt, None)
+
+            if winning_value:
+                config['auth'][p_opt.name.replace('-', '_')] = winning_value
+
+            return config
+
     def get_one_cloud(self, cloud=None, validate=True,
                       argparse=None, **kwargs):
         """Retrieve a single cloud configuration and merge additional options
@@ -208,15 +269,10 @@ class OpenStackConfig(object):
 
         args = self._fix_args(kwargs, argparse=argparse)
 
-        if cloud:
-            name = cloud
-        else:
-            name = 'openstack'
-
         if 'region_name' not in args:
-            args['region_name'] = self._get_region(name)
+            args['region_name'] = self._get_region(cloud)
 
-        config = self._get_base_cloud_config(name)
+        config = self._get_base_cloud_config(cloud)
 
         # Can't just do update, because None values take over
         for (key, val) in args.iteritems():
@@ -228,20 +284,8 @@ class OpenStackConfig(object):
                 if type(config[key]) is not bool:
                     config[key] = get_boolean(config[key])
 
-        if validate:
-            for key in REQUIRED_VALUES:
-                if key not in config or not config[key]:
-                    raise exceptions.OpenStackConfigException(
-                        'Unable to find full auth information for cloud'
-                        ' {name} in config files {files}'
-                        ' or environment variables.'.format(
-                            name=name, files=','.join(self._config_files)))
-            if 'project_name' not in config and 'project_id' not in config:
-                raise exceptions.OpenStackConfigException(
-                    'Neither project_name or project_id information found'
-                    ' for cloud {name} in config files {files}'
-                    ' or environment variables.'.format(
-                        name=name, files=','.join(self._config_files)))
+        if validate and ksc_auth:
+            config = self._validate_auth(config)
 
         # If any of the defaults reference other values, we need to expand
         for (key, value) in config.items():
@@ -249,7 +293,7 @@ class OpenStackConfig(object):
                 config[key] = value.format(**config)
 
         return cloud_config.CloudConfig(
-            name=name, region=config['region_name'], config=config)
+            name=cloud, region=config['region_name'], config=config)
 
 if __name__ == '__main__':
     config = OpenStackConfig().get_all_clouds()
